"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .accountbalance import AccountBalance, AccountBalanceTypedDict
from .commerceaddress import CommerceAddress, CommerceAddressTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from .weblink import WebLink, WebLinkTypedDict
from codat_lending.types import BaseModel
import pydantic
from typing import Dict, List, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class CommerceCompanyInfoTypedDict(TypedDict):
    r"""In the Codat system, company profile includes standard commercial details about
    a linked company, such as their address, phone number, and company registration.

    Explore our [data coverage](https://knowledge.codat.io/supported-features/commerce?view=tab-by-data-type&dataType=companyInfo) for this data type.

    """

    account_balances: NotRequired[List[AccountBalanceTypedDict]]
    r"""The available and current cash balances for the company's accounts"""
    addresses: NotRequired[List[CommerceAddressTypedDict]]
    r"""Addresses associated with the company"""
    base_currency: NotRequired[str]
    r"""The currency data type in Codat is the [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217) currency code, e.g. _GBP_.

    ## Unknown currencies

    In line with the ISO 4217 specification, the code _XXX_ is used when the data source does not return a currency for a transaction.

    There are only a very small number of edge cases where this currency code is returned by the Codat system.
    """
    commerce_platform_ref: NotRequired[str]
    r"""Identifier or reference for the company in the commerce software"""
    company_legal_name: NotRequired[str]
    r"""The full legal name of the company"""
    company_name: NotRequired[str]
    r"""The name of the company"""
    created_date: NotRequired[str]
    r"""In Codat's data model, dates and times are represented using the <a class=\"external\" href=\"https://en.wikipedia.org/wiki/ISO_8601\" target=\"_blank\">ISO 8601 standard</a>. Date and time fields are formatted as strings; for example:

    ```
    2020-10-08T22:40:50Z
    2021-01-01T00:00:00
    ```



    When syncing data that contains `DateTime` fields from Codat, make sure you support the following cases when reading time information:

    - Coordinated Universal Time (UTC): `2021-11-15T06:00:00Z`
    - Unqualified local time: `2021-11-15T01:00:00`
    - UTC time offsets: `2021-11-15T01:00:00-05:00`

    > Time zones
    >
    > Not all dates from Codat will contain information about time zones.
    > Where it is not available from the underlying platform, Codat will return these as times local to the business whose data has been synced.
    """
    modified_date: NotRequired[str]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    r"""Phone numbers associated with the company"""
    registration_number: NotRequired[str]
    r"""The registration number of the company"""
    source_modified_date: NotRequired[str]
    source_urls: NotRequired[Dict[str, str]]
    r"""URL addresses for the originating system. For example, potential use cases include 'deeplinking' to the originating system"""
    web_links: NotRequired[List[WebLinkTypedDict]]
    r"""Weblinks associated with the company"""


class CommerceCompanyInfo(BaseModel):
    r"""In the Codat system, company profile includes standard commercial details about
    a linked company, such as their address, phone number, and company registration.

    Explore our [data coverage](https://knowledge.codat.io/supported-features/commerce?view=tab-by-data-type&dataType=companyInfo) for this data type.

    """

    account_balances: Annotated[
        Optional[List[AccountBalance]], pydantic.Field(alias="accountBalances")
    ] = None
    r"""The available and current cash balances for the company's accounts"""

    addresses: Optional[List[CommerceAddress]] = None
    r"""Addresses associated with the company"""

    base_currency: Annotated[Optional[str], pydantic.Field(alias="baseCurrency")] = None
    r"""The currency data type in Codat is the [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217) currency code, e.g. _GBP_.

    ## Unknown currencies

    In line with the ISO 4217 specification, the code _XXX_ is used when the data source does not return a currency for a transaction.

    There are only a very small number of edge cases where this currency code is returned by the Codat system.
    """

    commerce_platform_ref: Annotated[
        Optional[str], pydantic.Field(alias="commercePlatformRef")
    ] = None
    r"""Identifier or reference for the company in the commerce software"""

    company_legal_name: Annotated[
        Optional[str], pydantic.Field(alias="companyLegalName")
    ] = None
    r"""The full legal name of the company"""

    company_name: Annotated[Optional[str], pydantic.Field(alias="companyName")] = None
    r"""The name of the company"""

    created_date: Annotated[Optional[str], pydantic.Field(alias="createdDate")] = None
    r"""In Codat's data model, dates and times are represented using the <a class=\"external\" href=\"https://en.wikipedia.org/wiki/ISO_8601\" target=\"_blank\">ISO 8601 standard</a>. Date and time fields are formatted as strings; for example:

    ```
    2020-10-08T22:40:50Z
    2021-01-01T00:00:00
    ```



    When syncing data that contains `DateTime` fields from Codat, make sure you support the following cases when reading time information:

    - Coordinated Universal Time (UTC): `2021-11-15T06:00:00Z`
    - Unqualified local time: `2021-11-15T01:00:00`
    - UTC time offsets: `2021-11-15T01:00:00-05:00`

    > Time zones
    >
    > Not all dates from Codat will contain information about time zones.
    > Where it is not available from the underlying platform, Codat will return these as times local to the business whose data has been synced.
    """

    modified_date: Annotated[Optional[str], pydantic.Field(alias="modifiedDate")] = None

    phone_numbers: Annotated[
        Optional[List[PhoneNumber]], pydantic.Field(alias="phoneNumbers")
    ] = None
    r"""Phone numbers associated with the company"""

    registration_number: Annotated[
        Optional[str], pydantic.Field(alias="registrationNumber")
    ] = None
    r"""The registration number of the company"""

    source_modified_date: Annotated[
        Optional[str], pydantic.Field(alias="sourceModifiedDate")
    ] = None

    source_urls: Annotated[
        Optional[Dict[str, str]], pydantic.Field(alias="sourceUrls")
    ] = None
    r"""URL addresses for the originating system. For example, potential use cases include 'deeplinking' to the originating system"""

    web_links: Annotated[Optional[List[WebLink]], pydantic.Field(alias="webLinks")] = (
        None
    )
    r"""Weblinks associated with the company"""
